/*
 * This file is part of libtqtvd_astronav
 *
 * Copyright (C) 2008-2017 TQTVD Software
 * Copyright (C) 2017 Mirakulo Software (mirakulo@mirakulo.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

/**
 * @file  BCGraphicsDeviceGraphicManager.h
 *
 * AstroNav TQTVD Header of BCGraphicsDeviceGraphicManager class.
 *
 */

#ifndef BCGRAPHICSDEVICEASTROTV_H
#define BCGRAPHICSDEVICEASTROTV_H

#include <map>

#include <graphics/BIGraphicsDevice.h>
#include "BTWidget.h"
#include <pthread.h>
#include <graphics/IntSize.h>
#include <graphics/IntRect.h>

#include "IGraphicManagerAdapter.h"


using namespace TQTVD::GraphicManager ;

namespace BAL {

class BCGraphicsDeviceGraphicManager : public BIGraphicsDevice {
public:
	BCGraphicsDeviceGraphicManager();
	~BCGraphicsDeviceGraphicManager();

	/**
	  *
	  */
	virtual void addUpdateListener(BIListener* listener);
	virtual void removeUpdateListener(BIListener* listener);

	/**
	  *
	  */
	virtual void initialize(uint16_t width, uint16_t height, uint8_t depth);
	/**
	  *
	  */
	virtual void finalize();
	/**
	  *
	  */
	virtual BINativeImage* createNativeImage(const WebCore::IntSize size);
	virtual BINativeImage* createNativeImage(RGBA32Array&, const WebCore::IntSize size);

	/**
	  *
	  */
	virtual void copy(const BTWidget&, const BINativeImage&, WebCore::IntRect, WebCore::IntPoint, const uint8_t alphaChannel);
	/**
	  *
	  */
	virtual void drawArc(const BTWidget&, const WebCore::IntRect rect, uint16_t startAngle, uint16_t angleSpan, const WebCore::Color);
	/**
	  *
	  */
	virtual void drawEllipse(const BTWidget&, WebCore::IntRect, const WebCore::Color);
	/**
	  *
	  */
	virtual void drawLine(const BTWidget&, WebCore::IntPoint, WebCore::IntPoint, const WebCore::Color);
	/**
	 *
	 */
	virtual void setLineStyle(const BTWidget&, const int);

	/**
	  *
	  */
	virtual void drawRect(const BTWidget&, WebCore::IntRect, const WebCore::Color);
	/**
	  *
	  */
	virtual void fillConvexPolygon(const BTWidget&, size_t numPoints, const WebCore::IntPoint*, const WebCore::Color);
	/**
	  *
	  */
	virtual void fillRect(const BTWidget&, WebCore::IntRect, const WebCore::Color);
	/**
	  *
	  */
	virtual void setClip(const BTWidget&, WebCore::IntRect);
	/**
	  *
	  */
	virtual WebCore::IntRect clip(const BTWidget&);
	/**
	  *
	  */
	virtual void stretchBlit(const BTWidget&, const BINativeImage&, WebCore::IntRect src, WebCore::IntRect dst, const uint8_t alphaChannel);
	virtual WebCore::IntSize size() const;
	virtual void update(const BTWidget&, const WebCore::IntRect);
	virtual void clear(const BTWidget&, const WebCore::IntRect);
public:
	void *	setSuperInterface	 (void* si);
	void *	getSuperInterface	 (void) { return (void *) m_gm; }
	void *	setPrimarySurface (void* ps);
	void *	getPrimarySurface (void) { return (void *) m_screen; }

	void setScreenLessMode(bool mode);

	void             setBackgroundColor(WebCore::RGBA32 color);
	WebCore::RGBA32  getBackgroundColor(void);

private:
	inline void drawArc(const BTWidget&, const WebCore::Color, int zone, int xc, int yc, float& x0, float& y0, float x1, float y1, bool doSwap = true);
	IGraphicManagerAdapterSurface* applyTransparency(const BINativeImage&, const uint8_t alphaChannel);

	/* the super interface */
	IGraphicManagerAdapter * m_gm;
	bool m_ownAstro;

	/* the primary surface (surface of primary layer) */
	IGraphicManagerAdapterSurface* m_screen;
	bool              m_ownScreen;
	bool              m_screenLessMode;

	int m_width;
	int m_height;
	int m_depth;

	pthread_mutex_t  m_listenersMutex;
	std::map<const BTWidget *, BIListener*> m_listenerMap;

	WebCore::RGBA32  m_backgroundColor;

#ifndef NDEBUG
	bool             m_layoutTests;
#endif
};

} // namespace BAL

#endif // BCGRAPHICSDEVICEASTROTV_H
