/*
 * Copyright (C) 2007 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /**
 * @file BIEvent.h
 *
 * Any Event handled by OWBAL
 */
#ifndef BIEVENT_H_
#define BIEVENT_H_

namespace BAL {

    class BIWindowEvent;
    class BIInputEvent;
    class BIKeyboardEvent;
    class BIMouseEvent;
    class BIWheelEvent;
    class BITimerEvent;
    class BIPlayerEvent; // used for AstroNavPlayer

    
    /**
    * @brief the BIEvent
    * 
    * The event, base class of all events
    * 
    * @see BIEventLoop, BIInputEvent, BIWindowEvent
    */
    class BIEvent {
        public:
        // this is mandatory
            /**
            * BIEvent destructor
            */
            virtual ~BIEvent() {};

            /**
            * clone BIEvent
            */
            virtual BIEvent* clone() const = 0;
            
            /**
            * test if the event is a windowEvent
            */
            virtual BIWindowEvent* queryIsWindowEvent() { return 0; }
            /**
            * test if the event is a inputEvent
            */
            virtual BIInputEvent* queryIsInputEvent() { return 0; }
            /**
            * test if the event is a keyboardEvent
            */
            virtual BIKeyboardEvent* queryIsKeyboardEvent() { return 0; }
            /**
            * test if the event is a mouseEvent
            */
            virtual BIMouseEvent* queryIsMouseEvent() { return 0; }
            /**
            * test if the event is a wheelEvent
            */
            virtual BIWheelEvent* queryIsWheelEvent() { return 0; }
            /**
            * test if the event is a timerEvent
            */
            virtual BITimerEvent* queryIsTimerEvent() { return 0; }
            /**
            * test if the event is a playerEvent
            */
            virtual BIPlayerEvent* queryIsPlayerEvent() { return 0; }
    };

}

#define IMPLEMENT_BIEVENT  \
    public:


#endif // BIEVENT_H
