/*
 * Copyright (C) 2007 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @file BIWheelEvent.h
 *
 * Interface of Wheel Event
 */

#ifndef BIWheelEvent_h
#define BIWheelEvent_h

#include <graphics/IntPoint.h>
#include "BIInputEvent.h"
using WebCore::IntPoint;

namespace BAL {

    /**
    * @brief the WheelEvent
    *
    * The wheel event (you know ? it's on the mouse between the two buttons)
    *
    * @see BIEvent, BIEventLoop
    */
    class BIWheelEvent : public BIInputEvent {
    public:
        /**
        * BIWheelEvent destructor
        */
        virtual ~BIWheelEvent() {}

        /**
        * get position
        */
        virtual const IntPoint& pos() const = 0;
        /**
        * get global position
        */
        virtual const IntPoint& globalPos() const = 0;
        /**
        * get delta X
        */
        virtual int deltaX() const = 0;
        /**
        * get delta Y
        */
        virtual int deltaY() const = 0;
        /**
        * return if it's accepted
        */
        virtual bool isAccepted() const = 0;

        /**
        * force accept
        */
        virtual void accept() = 0;
        /**
        * force ignore
        */
        virtual void ignore() = 0;

        /**
        * return if it's continuous
        */
        virtual bool isContinuous() const = 0;

        /**
        * return the continuous delata X
        */
        virtual float continuousDeltaX() const = 0;

        /**
        * return the continuous delata Y
        */
        virtual float continuousDeltaY() const = 0;

        /**
        * BIWheelEvent clone
        */
        virtual BIEvent* clone() const = 0;

        virtual BIWheelEvent* queryIsWheelEvent() { return this; }
    };
}

#endif
