/*
 * This file is part of the WebKit project.
 *
 * Copyright (C) 2006 Apple Computer, Inc.
 * Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com
 * All rights reserved.
 * Copyright (C) 2007 Pleyo. All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#ifndef RenderThemeBal_H_
#define RenderThemeBal_H_

#include "RenderTheme.h"
#include <gtk/gtk.h>
#include <gtk/gtkimcontext.h>
#include <gtk/gtkimmulticontext.h>

namespace WebCore
{

struct ThemeData
{
    ThemeData() :m_part(0), m_state(0) {}

    unsigned m_part;
    unsigned m_state;
};

class RenderThemeBal : public RenderTheme
{
public:
    RenderThemeBal();
    ~RenderThemeBal();

    // A method asking if the theme's controls actually care about redrawing when hovered.
    virtual bool supportsHover(const RenderStyle*) const { return false; }

    // Methods for each appearance value.
    virtual bool paintCheckbox(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r);
    virtual void setCheckboxSize(RenderStyle* /*style*/) const;

    virtual bool paintRadio(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r);
    virtual void setRadioSize(RenderStyle* /*style*/) const;

    virtual void adjustButtonStyle(CSSStyleSelector* selector, RenderStyle* style, Element* e) const;
    virtual bool paintButton(RenderObject* /*o*/, const RenderObject::PaintInfo& /*i*/, const IntRect& /*r*/);

    virtual void adjustTextFieldStyle(CSSStyleSelector* selector, RenderStyle* style, Element* e) const;
    virtual bool paintTextField(RenderObject* /*o*/, const RenderObject::PaintInfo& /*i*/, const IntRect& /*r*/);

    virtual bool paintTextArea(RenderObject* /*o*/, const RenderObject::PaintInfo& /*i*/, const IntRect& /*r*/);

    virtual bool paintMenuList(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r);

    virtual bool paintMenuListButton(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r);

    virtual void systemFont(int propId, FontDescription&) const;

    virtual bool isControlStyled(const RenderStyle* style, const BorderData&, const BackgroundLayer&, const Color& backgroundColor) const;

private:
    void addIntrinsicMargins(RenderStyle* style) const;
    void close();

    unsigned determineState(RenderObject* o);
    bool supportsFocus(EAppearance appearance);

    ThemeData getThemeData(RenderObject* o);

    GtkIMContext *m_context;
    GdkWindow *m_window;
};

};

#endif /*RenderThemeBal_H_*/

